/**
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.streamnative.pulsar.handlers.kop.offset;

import lombok.Data;
import lombok.experimental.Accessors;

/**
 * Offset Metadata.
 */
@Data
@Accessors(fluent = true)
public class OffsetMetadata {

    public static final long INVALID_OFFSET = -1L;
    public static final String NO_METADATA = "";
    public static final OffsetMetadata INVALID_OFFSET_METADATA =
        new OffsetMetadata(INVALID_OFFSET, NO_METADATA);

    private final long offset;
    private final String metadata;

    public OffsetMetadata(long offset) {
        this(offset, NO_METADATA);
    }

    public OffsetMetadata(long offset, String metadata) {
        this.offset = offset;
        this.metadata = metadata;
    }

    @Override
    public String toString() {
        return String.format(
            "OffsetMetadata[%d,%s]",
            offset,
            metadata != null && metadata.length() > 0 ? metadata : "NO_METADATA"
        );
    }

}
